from abc import ABC, abstractmethod

from botik.input.keyboard.keyboard_markup import KeyboardMarkup
from botik.input.keyboard.markup_factory import KeyboardMarkupFactory
from botik.input.message_handlers.events.bot_events import BotEvents

from botik.api import api, get_api


class Page(ABC):

    def __init__(self, path: str, markup_factory: KeyboardMarkupFactory, data):
        self.path = path
        self.markup: KeyboardMarkup = None

        self._markup_factory = markup_factory
        self._data = data

        self._create_keyboard_markup()
        self._initialize()

    def _initialize(self):
        """
        Called during the instantiation phase.
        """
        pass

    def destruct(self):
        """
        Called when the user changes the page.
        Must be overriden, if page instance is subscribed to bot events.
        """
        pass

    def _create_keyboard_markup(self):
        """
        Construct the new self.markup object.
        """
        self.markup = self._markup_factory.create(inline=self._data.inline, one_time=self._data.one_time)

    async def send(self, user, message, markup=False, attachment=None):
        """
        Send a text message to user shorthand method.
        :param user: User object.
        :param message: Text message to send to user.
        :param markup: Should the keyboard be sent?
        """
        if not markup:
            await get_api().msg.send(user, message, attachment=attachment)
        else:
            await get_api().msg.send_with_keyboard(user, message, self.markup)

    def get_data(self):
        return self._data

    def get_back_path(self):
        """
        Get path of the previous page, when navigation's get_back method was called.
        Override to get a specific page path.
        :return: Page path string
        """
        return '..'

    async def _respond_to_input(self, user, text):
        """
        Called to handle user input text.
        Overridable.
        """
        pass

    @abstractmethod
    async def make_page_content(self, user):
        """
        This method is called when a user navigates to the page.
        Override it to send messages and keyboard.
        """
        pass

    async def handle_raw_input(self, user, text, only_check_press=False):
        """
        Called when user sends anything to bot while this page instance is active.
        Don't override.
        """
        pressed_button = self.markup.get_pressed_button(text)
        if pressed_button:
            await pressed_button.press(user)
        elif not only_check_press:
            await self._respond_to_input(user, text)
